@@ -27,7 +27,6 @@
 #endif
 
 #include "eog-preferences-dialog.h"
-#include "eog-plugin-manager.h"
 #include "eog-scroll-view.h"
 #include "eog-util.h"
 #include "eog-config-keys.h"
@@ -36,6 +35,7 @@
 #include <glib/gi18n.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
+#include <libpeas-gtk/peas-gtk-plugin-manager.h>
 
 #define EOG_PREFERENCES_DIALOG_GET_PRIVATE(object) \
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOG_TYPE_PREFERENCES_DIALOG, EogPreferencesDialogPrivate))
@@ -255,7 +255,7 @@ eog_preferences_dialog_constructor (GType type,
 	g_settings_bind (priv->fullscreen_settings, EOG_CONF_FULLSCREEN_SECONDS,
 			 seconds_spin, "value", G_SETTINGS_BIND_DEFAULT);
 
-        plugin_manager = eog_plugin_manager_new ();
+        plugin_manager = peas_gtk_plugin_manager_new (NULL);
 
         g_assert (plugin_manager != NULL);
 
