@@ -16,7 +16,9 @@
 #include "eog-marshal.h"
 #include "eog-scroll-view.h"
 #include "eog-debug.h"
+#if 0
 #include "uta.h"
+#endif
 #include "zoom.h"
 
 /* Maximum size of delayed repaint rectangles */
@@ -43,12 +45,6 @@
  */
 #define IMAGE_VIEW_ZOOM_MULTIPLIER 1.05
 
-/* States for automatically adjusting the zoom factor */
-typedef enum {
-	ZOOM_MODE_FIT,		/* Image is fitted to scroll view even if the latter changes size */
-	ZOOM_MODE_FREE		/* The image remains at its current zoom factor even if the scrollview changes size  */
-} ZoomMode;
-
 #if 0
 /* Progressive loading state */
 typedef enum {
@@ -85,6 +81,7 @@ enum {
 	PROP_TRANSP_COLOR,
 	PROP_TRANSPARENCY_STYLE,
 	PROP_USE_BG_COLOR,
+	PROP_ZOOM_MODE,
 	PROP_ZOOM_MULTIPLIER
 };
 
@@ -106,7 +103,7 @@ struct _EogScrollViewPrivate {
 	cairo_surface_t *surface;
 
 	/* zoom mode, either ZOOM_MODE_FIT or ZOOM_MODE_FREE */
-	ZoomMode zoom_mode;
+	EogZoomMode zoom_mode;
 
 	/* whether to allow zoom > 1.0 on zoom fit */
 	gboolean upscale;
@@ -120,10 +117,12 @@ struct _EogScrollViewPrivate {
 	/* Current scrolling offsets */
 	int xofs, yofs;
 
+#if 0
 	/* Microtile arrays for dirty region.  This represents the dirty region
 	 * for interpolated drawing.
 	 */
 	EogUta *uta;
+#endif
 
 	/* handler ID for paint idle callback */
 	guint idle_id;
@@ -174,6 +173,7 @@ static void view_on_drag_data_get_cb (GtkWidget *widget,
 				      GdkDragContext*drag_context,
 				      GtkSelectionData *data, guint info,
 				      guint time, gpointer user_data);
+static void _set_zoom_mode_internal (EogScrollView *view, EogZoomMode mode);
 
 #define EOG_SCROLL_VIEW_GET_PRIVATE(object) \
 	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOG_TYPE_SCROLL_VIEW, EogScrollViewPrivate))
@@ -309,9 +309,8 @@ update_scrollbar_values (EogScrollView *view)
 {
 	EogScrollViewPrivate *priv;
 	int scaled_width, scaled_height;
-	int xofs, yofs;
 	gdouble page_size,page_increment,step_increment;
-	gdouble lower, upper, value;
+	gdouble lower, upper;
 	GtkAllocation allocation;
 
 	priv = view->priv;
@@ -332,23 +331,19 @@ update_scrollbar_values (EogScrollView *view)
 		/* Set scroll bounds and new offsets */
 		lower = 0;
 		upper = scaled_width;
-		xofs = CLAMP (priv->xofs, 0, upper - page_size);
-		if (gtk_adjustment_get_value (priv->hadj) != xofs) {
-			value = xofs;
-			priv->xofs = xofs;
+		priv->xofs = CLAMP (priv->xofs, 0, upper - page_size);
 
-			g_signal_handlers_block_matched (
-				priv->hadj, G_SIGNAL_MATCH_DATA,
-				0, 0, NULL, NULL, view);
+		g_signal_handlers_block_matched (
+			priv->hadj, G_SIGNAL_MATCH_DATA,
+			0, 0, NULL, NULL, view);
 
-			gtk_adjustment_configure (priv->hadj, value, lower,
-						  upper, step_increment,
-						  page_increment, page_size);
+		gtk_adjustment_configure (priv->hadj, priv->xofs, lower,
+					  upper, step_increment,
+					  page_increment, page_size);
 
-			g_signal_handlers_unblock_matched (
-				priv->hadj, G_SIGNAL_MATCH_DATA,
-				0, 0, NULL, NULL, view);
-		}
+		g_signal_handlers_unblock_matched (
+			priv->hadj, G_SIGNAL_MATCH_DATA,
+			0, 0, NULL, NULL, view);
 	}
 
 	if (gtk_widget_get_visible (GTK_WIDGET (priv->vbar))) {
@@ -358,24 +353,19 @@ update_scrollbar_values (EogScrollView *view)
 
 		lower = 0;
 		upper = scaled_height;
-		yofs = CLAMP (priv->yofs, 0, upper - page_size);
+		priv->yofs = CLAMP (priv->yofs, 0, upper - page_size);
 
-		if (gtk_adjustment_get_value (priv->vadj) != yofs) {
-			value = yofs;
-			priv->yofs = yofs;
+		g_signal_handlers_block_matched (
+			priv->vadj, G_SIGNAL_MATCH_DATA,
+			0, 0, NULL, NULL, view);
 
-			g_signal_handlers_block_matched (
-				priv->vadj, G_SIGNAL_MATCH_DATA,
-				0, 0, NULL, NULL, view);
+		gtk_adjustment_configure (priv->vadj, priv->yofs, lower,
+					  upper, step_increment,
+					  page_increment, page_size);
 
-			gtk_adjustment_configure (priv->vadj, value, lower,
-						  upper, step_increment,
-						  page_increment, page_size);
-
-			g_signal_handlers_unblock_matched (
-				priv->vadj, G_SIGNAL_MATCH_DATA,
-				0, 0, NULL, NULL, view);
-		}
+		g_signal_handlers_unblock_matched (
+			priv->vadj, G_SIGNAL_MATCH_DATA,
+			0, 0, NULL, NULL, view);
 	}
 }
 
@@ -453,7 +443,7 @@ check_scrollbar_visibility (EogScrollView *view, GtkAllocation *alloc)
 			   width, height, bar_width, bar_height);
 
 	hbar_visible = vbar_visible = FALSE;
-	if (priv->zoom_mode == ZOOM_MODE_FIT)
+	if (priv->zoom_mode == EOG_ZOOM_MODE_SHRINK_TO_FIT)
 		hbar_visible = vbar_visible = FALSE;
 	else if (img_width <= width && img_height <= height)
 		hbar_visible = vbar_visible = FALSE;
@@ -1123,9 +1113,11 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 	GtkAllocation allocation;
 	int xofs, yofs;
 	GdkWindow *window;
+#if 0
 	int src_x, src_y;
 	int dest_x, dest_y;
 	int twidth, theight;
+#endif
 
 	priv = view->priv;
 
@@ -1163,7 +1155,7 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 	window = gtk_widget_get_window (GTK_WIDGET (priv->display));
 
 	/* Ensure that the uta has the full size */
-
+#if 0
 	twidth = (allocation.width + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
 	theight = (allocation.height + EOG_UTILE_SIZE - 1) >> EOG_UTILE_SHIFT;
 
@@ -1189,7 +1181,7 @@ scroll_to (EogScrollView *view, int x, int y, gboolean change_adjustments)
 		       dest_x, dest_y,
 		       allocation.width - abs (xofs),
 		       allocation.height - abs (yofs));
-
+#endif
 	/* Scroll the window area and process exposure synchronously. */
 
 	gdk_window_scroll (window, -xofs, -yofs);
@@ -1316,7 +1308,7 @@ set_zoom (EogScrollView *view, double zoom,
 	if (DOUBLE_EQUAL (priv->zoom, priv->min_zoom) && zoom < priv->zoom)
 		return;
 
-	priv->zoom_mode = ZOOM_MODE_FREE;
+	eog_scroll_view_set_zoom_mode (view, EOG_ZOOM_MODE_FREE);
 
 	gtk_widget_get_allocation (GTK_WIDGET (priv->display), &allocation);
 
@@ -1365,7 +1357,7 @@ set_zoom_fit (EogScrollView *view)
 
 	priv = view->priv;
 
-	priv->zoom_mode = ZOOM_MODE_FIT;
+	priv->zoom_mode = EOG_ZOOM_MODE_SHRINK_TO_FIT;
 
 	if (!gtk_widget_get_mapped (GTK_WIDGET (view)))
 		return;
@@ -1728,7 +1720,7 @@ display_size_change (GtkWidget *widget, GdkEventConfigure *event, gpointer data)
 	view = EOG_SCROLL_VIEW (data);
 	priv = view->priv;
 
-	if (priv->zoom_mode == ZOOM_MODE_FIT) {
+	if (priv->zoom_mode == EOG_ZOOM_MODE_SHRINK_TO_FIT) {
 		GtkAllocation alloc;
 
 		alloc.width = event->width;
@@ -2025,16 +2017,10 @@ update_pixbuf (EogScrollView *view, GdkPixbuf *pixbuf)
 static void
 image_changed_cb (EogImage *img, gpointer data)
 {
-	EogScrollViewPrivate *priv;
-
-	priv = EOG_SCROLL_VIEW (data)->priv;
-
 	update_pixbuf (EOG_SCROLL_VIEW (data), eog_image_get_pixbuf (img));
 
-	set_zoom_fit (EOG_SCROLL_VIEW (data));
-	check_scrollbar_visibility (EOG_SCROLL_VIEW (data), NULL);
-
-	gtk_widget_queue_draw (GTK_WIDGET (priv->display));
+	_set_zoom_mode_internal (EOG_SCROLL_VIEW (data),
+				 EOG_ZOOM_MODE_SHRINK_TO_FIT);
 }
 
 /*===================================
@@ -2066,7 +2052,7 @@ eog_scroll_view_set_zoom_upscale (EogScrollView *view, gboolean upscale)
 	if (priv->upscale != upscale) {
 		priv->upscale = upscale;
 
-		if (priv->zoom_mode == ZOOM_MODE_FIT) {
+		if (priv->zoom_mode == EOG_ZOOM_MODE_SHRINK_TO_FIT) {
 			set_zoom_fit (view);
 			gtk_widget_queue_draw (GTK_WIDGET (priv->display));
 		}
@@ -2251,7 +2237,7 @@ eog_scroll_view_zoom_out (EogScrollView *view, gboolean smooth)
 	set_zoom (view, zoom, FALSE, 0, 0);
 }
 
-void
+static void
 eog_scroll_view_zoom_fit (EogScrollView *view)
 {
 	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
@@ -2339,9 +2325,8 @@ eog_scroll_view_set_image (EogScrollView *view, EogImage *image)
 		if (priv->pixbuf == NULL) {
 			update_pixbuf (view, eog_image_get_pixbuf (image));
 			/* priv->progressive_state = PROGRESSIVE_NONE; */
-			set_zoom_fit (view);
-			check_scrollbar_visibility (view, NULL);
-			gtk_widget_queue_draw (GTK_WIDGET (priv->display));
+			_set_zoom_mode_internal (view,
+						 EOG_ZOOM_MODE_SHRINK_TO_FIT);
 
 		}
 #if 0
@@ -2431,9 +2416,9 @@ eog_scroll_view_init (EogScrollView *view)
 
 	priv->zoom = 1.0;
 	priv->min_zoom = MIN_ZOOM_FACTOR;
-	priv->zoom_mode = ZOOM_MODE_FIT;
+	priv->zoom_mode = EOG_ZOOM_MODE_SHRINK_TO_FIT;
 	priv->upscale = FALSE;
-	priv->uta = NULL;
+	//priv->uta = NULL;
 	priv->interp_type_in = CAIRO_FILTER_BILINEAR;
 	priv->interp_type_out = CAIRO_FILTER_BILINEAR;
 	priv->scroll_wheel_zoom = FALSE;
@@ -2553,10 +2538,12 @@ eog_scroll_view_dispose (GObject *object)
 	view = EOG_SCROLL_VIEW (object);
 	priv = view->priv;
 
+#if 0
 	if (priv->uta != NULL) {
 		eog_uta_free (priv->uta);
 		priv->uta = NULL;
 	}
+#endif
 
 	if (priv->idle_id != 0) {
 		g_source_remove (priv->idle_id);
@@ -2621,6 +2608,9 @@ eog_scroll_view_get_property (GObject *object, guint property_id,
 	case PROP_TRANSPARENCY_STYLE:
 		g_value_set_enum (value, priv->transp_style);
 		break;
+	case PROP_ZOOM_MODE:
+		g_value_set_enum (value, priv->zoom_mode);
+		break;
 	case PROP_ZOOM_MULTIPLIER:
 		g_value_set_double (value, priv->zoom_multiplier);
 		break;
@@ -2634,12 +2624,10 @@ eog_scroll_view_set_property (GObject *object, guint property_id,
 			      const GValue *value, GParamSpec *pspec)
 {
 	EogScrollView *view;
-	EogScrollViewPrivate *priv;
 
 	g_return_if_fail (EOG_IS_SCROLL_VIEW (object));
 
 	view = EOG_SCROLL_VIEW (object);
-	priv = view->priv;
 
 	switch (property_id) {
 	case PROP_ANTIALIAS_IN:
@@ -2666,6 +2654,9 @@ eog_scroll_view_set_property (GObject *object, guint property_id,
 	case PROP_TRANSPARENCY_STYLE:
 		eog_scroll_view_set_transparency (view, g_value_get_enum (value));
 		break;
+	case PROP_ZOOM_MODE:
+		eog_scroll_view_set_zoom_mode (view, g_value_get_enum (value));
+		break;
 	case PROP_ZOOM_MULTIPLIER:
 		eog_scroll_view_set_zoom_multiplier (view, g_value_get_double (value));
 		break;
@@ -2754,7 +2745,7 @@ eog_scroll_view_class_init (EogScrollViewClass *klass)
 	 * EogScrollView:transparency-color:
 	 *
 	 * This is the color used to fill the transparent parts of an image
-	 * if :transparency-style is set to use a custom color.
+	 * if #EogScrollView:transparency-style is set to %EOG_TRANSP_COLOR.
 	 */
 	g_object_class_install_property (
 		gobject_class, PROP_TRANSP_COLOR,
@@ -2774,6 +2765,13 @@ eog_scroll_view_class_init (EogScrollViewClass *klass)
 				   EOG_TRANSP_CHECKED,
 				   G_PARAM_READWRITE | G_PARAM_STATIC_NAME));
 
+	g_object_class_install_property (
+		gobject_class, PROP_ZOOM_MODE,
+		g_param_spec_enum ("zoom-mode", NULL, NULL,
+				   EOG_TYPE_ZOOM_MODE,
+				   EOG_ZOOM_MODE_SHRINK_TO_FIT,
+				   G_PARAM_READWRITE | G_PARAM_STATIC_NAME));
+
 	view_signals [SIGNAL_ZOOM_CHANGED] =
 		g_signal_new ("zoom_changed",
 			      EOG_TYPE_SCROLL_VIEW,
@@ -2953,7 +2951,7 @@ _eog_scroll_view_update_bg_color (EogScrollView *view)
 		priv->background_surface = NULL;
 	}
 
-	gtk_widget_modify_bg (GTK_WIDGET (view),
+	gtk_widget_modify_bg (GTK_WIDGET (priv->display),
 			      GTK_STATE_NORMAL,
 			      selected);
 }
@@ -3018,3 +3016,40 @@ eog_scroll_view_set_zoom_multiplier (EogScrollView *view,
 
 	g_object_notify (G_OBJECT (view), "zoom-multiplier");
 }
+
+/* Helper to cause a redraw even if the zoom mode is unchanged */
+static void
+_set_zoom_mode_internal (EogScrollView *view, EogZoomMode mode)
+{
+	gboolean notify = (mode != view->priv->zoom_mode);
+
+
+	if (mode == EOG_ZOOM_MODE_SHRINK_TO_FIT)
+		eog_scroll_view_zoom_fit (view);
+	else
+		view->priv->zoom_mode = mode;
+	
+	if (notify)
+		g_object_notify (G_OBJECT (view), "zoom-mode");
+}
+
+
+void
+eog_scroll_view_set_zoom_mode (EogScrollView *view, EogZoomMode mode)
+{
+	g_return_if_fail (EOG_IS_SCROLL_VIEW (view));
+
+	if (view->priv->zoom_mode == mode)
+		return;
+
+	_set_zoom_mode_internal (view, mode);
+}
+
+EogZoomMode
+eog_scroll_view_get_zoom_mode (EogScrollView *view)
+{
+	g_return_val_if_fail (EOG_IS_SCROLL_VIEW (view),
+			      EOG_ZOOM_MODE_SHRINK_TO_FIT);
+
+	return view->priv->zoom_mode;
+}
